! ******************************************************************
! ******************************************************************

subroutine acondg(n,mA,xA,lA,uA,equatnA,linearA,optprojA,jcnnzmaxA,hnnzmaxA,&
                    mB,xB,lB,uB,equatnB,linearB,optprojB,jcnnzmaxB,hnnzmaxB,&
                    erropt,iter,nprojA,nprojB,time,info)
                      
  use myproblem                    

  implicit none
  
  ! SCALARS ARGUMENTS
  integer,intent(in)  :: n,mA,mB,optprojA,optprojB,jcnnzmaxA,jcnnzmaxB,&
                         hnnzmaxA,hnnzmaxB,erropt
  integer,intent(out) :: iter,nprojA,nprojB,info
  real(kind=4),intent(out) :: time
	
  ! ARRAYS ARGUMENTS
  real(kind=8),intent(inout) :: xA(n),xB(n)
  real(kind=8),intent(in) :: lA(n),uA(n),lB(n),uB(n)
  logical,intent(in) :: equatnA(mA),linearA(mA),equatnB(mB),linearB(mB)
   
  ! LOCAL SCALARS
  integer :: i,set,maxiter,lack,lacklim,itCondG,infoProj,flag
  real(kind=8) :: a,b,angle,theta,lambda,gama,eps,tlmult,cfrac,tol,tolprog
  real(kind=4) :: timeini, timefin
  logical :: iprint
  
  ! LOCAL ARRAYS 
  real(kind=8) :: cA(mA),cB(mB),w(n),xAprev(n),xBprev(n),c(4),cprev(4)
  
  ! Start timing
	
  call cpu_time(timeini)	
  
  ! Initialize parameters
		
  maxiter   = 10000
  tol       = 1.0d-8
  tolprog   = 1.0d-8
  eps       = 1.0d-8
  theta     = min( norm2(xA-xB), 0.2d0 - eps )
  lambda    = min( norm2(xA-xB), 0.2d0 - eps )
  gama      = min( norm2(xA-xB), 0.1d0 - eps )
  
  lacklim = 2
  tlmult  = 1.0d-1
  cfrac   = 9.0d-1
  
  ! Print information?
  
  iprint = .true.
  	
  ! Counters
	
  iter   = 0
  lack   = 0
  nprojA = 0
  nprojB = 0
  
  if (iprint) then
	  write(*,1000)
	  write(*,1010) n
	  write(*,1020) tol
	  write(*,1030)
  end if  
  
  ! Compute the feasibility violation at xA with respect to set A
  
  set = 1
  
  c(1) = 0.0d0
  do i = 1,n
	 c(1) = max( c(1) , xA(i) - uA(i) )
	 c(1) = max( c(1) , lA(i) - xA(i) )
  end do
  
  do i = 1,mA
	  call evalc(n,xA,set,i,cA(i))
  end do 
  
  do i = 1,mA
     if ( equatnA(i) ) then
        c(1)  = max( c(1) , abs( cA(i) ) )
     else
        c(1)  = max( c(1) , cA(i) )
     end if
  end do
  
  ! Compute the feasibility violation at xA with respect to set B
  
  set = 2 
  
  c(2) = 0.0d0
  do i = 1,n
	 c(2) = max( c(2) , xA(i) - uB(i) )
	 c(2) = max( c(2) , lB(i) - xA(i) )
  end do
  
  do i = 1,mB
	  call evalc(n,xA,set,i,cB(i))
  end do
  
  do i = 1,mB
     if ( equatnB(i) ) then
        c(2)  = max( c(2) , abs( cB(i) ) )
     else
        c(2)  = max( c(2) , cB(i) )
     end if
  end do
  
  ! Check the feasibility of xA with respect to set A
  
  if ( c(1) > tol ) then
	info = - 1
	write(*,1031)
	return
  end if
  
  ! Test for stopping at xA
  
  if ( c(1) <= tol .and. c(2) <= tol ) then
	  info = 1
	  write(*,1040) iter,c(1),c(2)
	  
	  ! Stop timing
			
   	  call cpu_time(timefin)	
	  time = timefin - timeini
	
	  if ( iprint ) write(*,1050) time
		
	  return
  end if
  
  ! Compute the feasibility violations at xB with respect to set A
  
  set = 1
  
  c(3) = 0.0d0
  do i = 1,n
	 c(3) = max( c(3) , xB(i) - uA(i) )
	 c(3) = max( c(3) , lA(i) - xB(i) )
  end do
  
  do i = 1,mA
	  call evalc(n,xB,set,i,cA(i))
  end do
  
  do i = 1,mA
     if ( equatnA(i) ) then
        c(3)  = max( c(3) , abs( cA(i) ) )
     else
        c(3)  = max( c(3) , cA(i) )
     end if
  end do
  
  ! Compute the feasibility violations at xB with respect to set B
  
  set = 2 
  
  c(4) = 0.0d0
  do i = 1,n
	 c(4) = max( c(4) , xB(i) - uB(i) )
	 c(4) = max( c(4) , lB(i) - xB(i) )
  end do
  
  do i = 1,mB
	  call evalc(n,xB,set,i,cB(i))
  end do
  
  do i = 1,mB
     if ( equatnB(i) ) then
        c(4)  = max( c(4) , abs( cB(i) ) )
     else
        c(4)  = max( c(4) , cB(i) )
     end if
  end do 
  
  ! Check the feasibility of xB with respect to set B
  
  if ( c(4) > tol ) then
	info = - 1
	write(*,1032)
	return
  end if
  
  ! Test for stopping at xB
  
  if ( c(3) <= tol .and. c(4) <= tol ) then
	  info = 2
	  write(*,1060) iter,c(1),c(2),c(3),c(4)
	  
	  ! Stop timing
			
   	  call cpu_time(timefin)	
	  time = timefin - timeini
	
	  if ( iprint ) write(*,1050) time
	  
	  return
  end if

  !---------------------------------------------------------------------  
  !     Main loop
  !---------------------------------------------------------------------     

  Main_loop: do
  
    if ( iprint .and. iter > 0 .and. mod(iter,10) .eq. 0 ) write(*,1030)
	if ( iprint ) write(*,1060) iter,c(1),c(2),c(3),c(4)
	 
  
	! Stopping for lack of progress
		                    
	if ( iter > 0 .and.    &
	                    maxval( abs(xA-xAprev) ) <=  tolprog .and.       &
	                    maxval( abs(xB-xBprev) ) <=  tolprog ) then                    
	                    
	    lack = lack + 1
	    
	    if ( lack >= lacklim ) then
	    
			info = 3
			
			! Stop timing
				
			call cpu_time(timefin)	
			time = timefin - timeini
			
			if ( iprint ) write(*,1070) time
			return
		end if
	else
		lack = 0
	end if
    
    ! Test whether the number of iterations is exhausted

	if ( iter >= maxiter ) then
		info = 4
		
		! Stop timing
			
		call cpu_time(timefin)	
		time = timefin - timeini
		
		if ( iprint ) write(*,1080) time
		return
	end if

	! Prepare the iteration
	
	if ( iter > 0 .and. c(2) > cfrac * cprev(2) .and. c(3) > cfrac * cprev(3) ) then	
		gama   = tlmult * gama
		theta  = tlmult * theta
		lambda = tlmult * lambda
	end if
		
	! Save previous data
	
	xAprev = xA
	xBprev = xB
	cprev = c
	
	! Iterate
	
	iter = iter + 1

	! Compute the projection of xA onto B with respect to xB
	
	nprojB = nprojB + 1
	
	set = 2
		
	if ( optprojB == 0 ) then
		call exactprojection(n,set,xA,w,infoProj)
	elseif ( optprojB == 1 ) then
		call CondG(n,set,xB,xA,w,mB,lB,uB,equatnB,linearB,jcnnzmaxB,hnnzmaxB,&
		           gama,theta,lambda,erropt,itCondG,infoProj)
	elseif ( optprojB == 2 ) then
		call proj(n,set,xA,xB,w,mB,lB,uB,equatnB,linearB,jcnnzmaxB,hnnzmaxB,infoProj)
	else
		info = -1
			
		! Stop timing
		
		call cpu_time(timefin)	
		time = timefin - timeini
		
		if ( iprint ) write(*,1081) time
		return
	end if
	
	if ( infoProj /= 0 ) then
		info = -1
		
		! Stop timing
		
		call cpu_time(timefin)	
		time = timefin - timeini
		
		if ( iprint .and. optprojB == 0 ) write(*,1090) time
		if ( iprint .and. optprojB == 1 ) write(*,1100) time
		if ( iprint .and. optprojB == 2 ) write(*,1110) time
		return
	end if	
	
	xB = w
	
	! Compute the feasibility violation at xB with respect to set A
  
	set = 1
	
	c(3) = 0.0d0
	do i = 1,n
		c(3) = max( c(3) , xB(i) - uA(i) )
		c(3) = max( c(3) , lA(i) - xB(i) )
	end do

	do i = 1,mA
	  call evalc(n,xB,set,i,cA(i))
	end do

	do i = 1,mA
		if ( equatnA(i) ) then
			c(3)  = max( c(3) , abs( cA(i) ) )
		else
			c(3)  = max( c(3) , cA(i) )
		end if
	end do

	! Compute the feasibility violation at xB with respect to set B

	set = 2 
	
	c(4) = 0.0d0
	do i = 1,n
		c(4) = max( c(4) , xB(i) - uB(i) )
		c(4) = max( c(4) , lB(i) - xB(i) )
	end do

	do i = 1,mB
	  call evalc(n,xB,set,i,cB(i))
	end do

	do i = 1,mB
		if ( equatnB(i) ) then
			c(4)  = max( c(4) , abs( cB(i) ) )
		else
			c(4)  = max( c(4) , cB(i) )
		end if
	end do 
	
	! Test for stopping at xB
  
	if ( c(3) <= tol .and. c(4) <= tol ) then
		info = 2
		if ( iprint ) write(*,1040) iter,c(3),c(4)

		! Stop timing
			
		call cpu_time(timefin)	
		time = timefin - timeini

		if ( iprint ) write(*,1050) time

		return
	end if
    
	! Compute the projection of xB onto A with respect to xA
	
	nprojA = nprojA + 1
	
	set = 1
	
	if ( optprojA == 0 ) then
		call exactprojection(n,set,xB,w,infoProj)
	elseif ( optprojA == 1 ) then
		call CondG(n,set,xA,xB,w,mA,lA,uA,equatnA,linearA,jcnnzmaxA,hnnzmaxA,&
		                gama,theta,lambda,erropt,itCondG,infoProj)
	elseif ( optprojA == 2 ) then			
		call proj(n,set,xB,xA,w,mA,lA,uA,equatnA,linearA,jcnnzmaxA,hnnzmaxA,infoProj)
	else
		info = -1
			
		! Stop timing
		
		call cpu_time(timefin)	
		time = timefin - timeini
		
		if ( iprint ) write(*,1081) time
		return
	end if
	
	if ( infoProj /= 0 ) then
		info = -1
		
		! Stop timing
		
		call cpu_time(timefin)	
		time = timefin - timeini
		
		if ( iprint .and. optprojB == 0 ) write(*,1090) time
		if ( iprint .and. optprojB == 1 ) write(*,1100) time
		if ( iprint .and. optprojB == 2 ) write(*,1110) time
		return
	end if	
		
	xA = w
	
	! Compute the feasibility violation at xA with respect to set A
  
	set = 1
	
	c(1) = 0.0d0
	do i = 1,n
		c(1) = max( c(1) , xA(i) - uA(i) )
		c(1) = max( c(1) , lA(i) - xA(i) )
	end do

	do i = 1,mA
		call evalc(n,xA,set,i,cA(i))
	end do

	do i = 1,mA
		if ( equatnA(i) ) then
			c(1)  = max( c(1) , abs( cA(i) ) )
		else
			c(1)  = max( c(1) , cA(i) )
		end if
	end do

	! Compute the feasibility violation at xA with respect to set B

	set = 2 
	
	c(2) = 0.0d0
	do i = 1,n
		c(2) = max( c(2) , xA(i) - uB(i) )
		c(2) = max( c(2) , lB(i) - xA(i) )
	end do

	do i = 1,mB
		call evalc(n,xA,set,i,cB(i))
	end do

	do i = 1,mB
		if ( equatnB(i) ) then
			c(2)  = max( c(2) , abs( cB(i) ) )
		else
			c(2)  = max( c(2) , cB(i) )
		end if
	end do
	
	! Test for stopping at xA
  
	if ( c(1) <= tol .and. c(2) <= tol ) then
	    info = 1
	    write(*,1040) iter,c(1),c(2)
	      
	  ! Stop timing
			
	    call cpu_time(timefin)	
	    time = timefin - timeini

	    if ( iprint ) write(*,1050) time
	 	
	    return
	end if	

  end do Main_loop

  !--------------------------------------------------------------------- 
  !     End of main loop
  !---------------------------------------------------------------------
  
  ! Non-executable statements
	
	1000 format(/,1X,'-----------------------------------------------------------------------',/,1X,& 
					 'ACondG: Alternating conditional gradient method for feasibility problem',/,1X,& 
					 '-----------------------------------------------------------------------')
	1010 format(/,1X,'Number of variables:',1X,I5)	
	
	1020 format(/,1X,'Feasibility tolerance:',1X,1P,D7.1)
	
	1030 format(/,4X,'it',5X,'A(xA)',6X,'B(xA)',6X,'A(xB)',6X,'B(xB)')
	
	1031 format(/,1X,'Flag of ACondG: the first iterate xA must be feasible for set A')	
	
	1032 format(/,1X,'Flag of ACondG: the first iterate xB must be feasible for set B')	
		
	1040 format(1X,I5,6X,'-',10X,'-',7X,1P,D8.2,3X,1P,D8.2)
	
	1050 format(/,1X,'Flag of ACondG: a feasible point was found',/,/,1X, &
					 'Total CPU time in seconds: ',0P,F8.2)		
	
	1060 format(1X,I5,4(3X,1P,D8.2))
	
	1070 format(/,1X,'Flag of ACondG: no progress is verified. The problem may be',/,&
	                 '                 infeasible. Further analysis is required.',/,/,1X,&
					  'Total CPU time in seconds: ',0P,F8.2)

	1080 format(/,1X,'Flag of ACondG: maximum of iterations reached',/,/,1X,&
				   	 'Total CPU time in seconds: ',0P,F8.2)

	1081 format(/,1X,'Flag of ACondG: invalid projection option',/,/,1X,&
				   	 'Total CPU time in seconds: ',0P,F8.2)	 				   	 
				   	 
	1090 format(/,1X,'Mandatory routine exactprojection not coded by the user.',/,/,1X,&
				   	 'Total CPU time in seconds: ',0P,F8.2)	   	 
	
	1100 format(/,1X,'Flag of ACondG: An error occurred while running the CondG method.',/,/,1X,&
				   	 'Total CPU time in seconds: ',0P,F8.2)	  
				   
	1110 format(/,1X,'Flag of ACondG: An error occurred while running the Algencan method.',/,/,1X,&
				   	 'Total CPU time in seconds: ',0P,F8.2)	  	 

end subroutine acondg
