! ******************************************************************
! ******************************************************************

subroutine evalz(n,set,y,w,z,m,l,u,equatn,linear,jcnnzmax,hnnzmax,inform)

  use globals
  
  implicit none
  
  ! SCALAR ARGUMENTS
  integer, intent(in)  :: n,m,set,jcnnzmax,hnnzmax
  integer, intent(out) :: inform
  
  ! ARRAY ARGUMENTS
  real(kind=8), intent(in)  :: y(n),w(n),l(n),u(n)
  real(kind=8), intent(out) :: z(n)
  logical,intent(in) :: equatn(m),linear(m)
  

  ! LOCAL SCALARS
  logical      :: checkder
  integer      :: allocerr,nvparam,alinfo
  real(kind=8) :: cnorm,efacc,efstain,eoacc,eostain,epsfeas,epsopt, &
       f,nlpsupn,snorm

  ! LOCAL ARRAYS
  character(len=15)     :: strtmp
  character(len=80)     :: specfnm,outputfnm,vparam(10)
  logical               :: coded(11)
  real(kind=8), pointer :: lambda(:),x(:)

  ! EXTERNAL SUBROUTINES
  external :: myevalf,myevalg,myevalh,myevalc,myevaljac,myevalhc, &
              myevalfc,myevalgjac,myevalgjacp,myevalhl,myevalhlp
      
  ! inform:
  !
  ! 0: Feasibility, optimality and complementarity satisfied
  ! 2: Too large penalty parameter. Infeasible problem?
  ! 3: Maximum number of algencan iterations reached
  
  yinner   = y
  winner   = w
  setinner = set

  ! Set lower bounds, upper bounds, and initial guess
  
  allocate(x(n),stat=allocerr)
  if ( allocerr .ne. 0 ) then
     write(*,*) 'Allocation error in main program'
     stop
  end if

  x = w

  ! Constraints

  allocate(lambda(m),stat=allocerr)
  if ( allocerr .ne. 0 ) then
     write(*,*) 'Allocation error in main program'
     stop
  end if

  lambda(1:m) = 0.0d0
	
  ! Coded subroutines

  coded(1:6)  = .true.  ! fsub, gsub, hsub, csub, jacsub, hcsub
  coded(7:11) = .false. ! fcsub,gjacsub,gjacpsub,hlsub,hlpsub

  ! Upper bounds on the number of sparse-matrices non-null elements
  
  ! Checking derivatives?

  checkder = .false.

  ! Parameters setting
  
  epsfeas   = 1.0d-08
  epsopt    = 1.0d-08

  efstain   = sqrt( epsfeas )
  eostain   = epsopt ** 1.5d0

  efacc     = sqrt( epsfeas )
  eoacc     = sqrt( epsopt )

  outputfnm = ''
  specfnm   = ''

  nvparam = 2
  vparam(1) = 'ITERATIONS-OUTPUT-DETAIL 00'
  vparam(2) = 'OUTER-ITERATIONS-LIMIT 100'
  

  ! Call Algencan

  ! E. G. Birgin and J. M. Martı́nez, Practical augmented Lagrangian 
  ! methods for constrained optimization, SIAM, 2014.
  
    
  call algencan(myevalf,myevalg,myevalh,myevalc,myevaljac,myevalhc, &
       myevalfc,myevalgjac,myevalgjacp,myevalhl,myevalhlp,jcnnzmax, &
       hnnzmax,epsfeas,epsopt,efstain,eostain,efacc,eoacc,outputfnm, &
       specfnm,nvparam,vparam,n,x,l,u,m,lambda,equatn,linear,coded, &
       checkder,f,cnorm,snorm,nlpsupn,inform,alinfo)
       
  inform = alinfo     
         
  z = x     

  deallocate(x,lambda,stat=allocerr)

  if ( allocerr .ne. 0 ) then
     write(*,*) 'Deallocation error in main program'
     stop
  end if

end subroutine evalz

! ******************************************************************
! ******************************************************************

subroutine myevalf(n,x,f,flag)

  use globals, only: y => yinner, w => winner

  implicit none

  ! SCALAR ARGUMENTS
  integer,      intent(in)  :: n
  integer,      intent(out) :: flag
  real(kind=8), intent(out) :: f

  ! ARRAY ARGUMENTS
  real(kind=8), intent(in) :: x(n)

  flag = 0	
      
  f = dot_product( w - y, x - w )

end subroutine myevalf

! ******************************************************************
! ******************************************************************

subroutine myevalg(n,x,g,flag)

  use globals, only: y => yinner, w => winner

  implicit none

  ! SCALAR ARGUMENTS
  integer,      intent(in)  :: n
  integer,      intent(out) :: flag

  ! ARRAY ARGUMENTS
  real(kind=8), intent(in)  :: x(n)
  real(kind=8), intent(out) :: g(n)
  
  ! LOCAL SCALAR
  integer :: i

  flag = 0
    	
  g = w - y

end subroutine myevalg

! ******************************************************************
! ******************************************************************

subroutine myevalh(n,x,hrow,hcol,hval,hnnz,lim,lmem,flag)

  implicit none

  ! SCALAR ARGUMENTS
  logical,      intent(out) :: lmem
  integer,      intent(in)  :: lim,n
  integer,      intent(out) :: flag,hnnz

  ! ARRAY ARGUMENTS
  integer,      intent(out) :: hcol(lim),hrow(lim)
  real(kind=8), intent(in)  :: x(n)
  real(kind=8), intent(out) :: hval(lim)
  
  ! LOCAL SCALAR
  integer :: i,j,k

  flag = 0
  lmem = .false.

  hnnz = 0
		
end subroutine myevalh

! ******************************************************************
! ******************************************************************

subroutine myevalc(n,x,ind,c,flag)

  use globals, only: set => setinner
  use myproblem
	
  implicit none

  ! SCALAR ARGUMENTS
  integer,      intent(in)  :: ind,n
  integer,      intent(out) :: flag
  real(kind=8), intent(out) :: c

  ! ARRAY ARGUMENTS
  real(kind=8), intent(in)  :: x(n)
  
  flag = 0

  call evalc(n,x,set,ind,c)
  
end subroutine myevalc

! ******************************************************************
! ******************************************************************

subroutine myevaljac(n,x,ind,jcvar,jcval,jcnnz,lim,lmem,flag)

  use globals, only: set => setinner
  use myproblem

  implicit none

  ! SCALAR ARGUMENTS
  logical, intent(out) :: lmem
  integer, intent(in)  :: ind,lim,n
  integer, intent(out) :: flag,jcnnz

  ! ARRAY ARGUMENTS
  integer,      intent(out) :: jcvar(lim)
  real(kind=8), intent(in)  :: x(n)
  real(kind=8), intent(out) :: jcval(lim)
  

  call evaljac(n,x,set,ind,jcvar,jcval,jcnnz,lim,lmem,flag)


end subroutine myevaljac

! ******************************************************************
! ******************************************************************

subroutine myevalhc(n,x,ind,hcrow,hccol,hcval,hcnnz,lim,lmem,flag)

  use globals, only: set => setinner
  use myproblem
  
  implicit none

  ! SCALAR ARGUMENTS
  logical, intent(out) :: lmem
  integer, intent(in)  :: ind,lim,n
  integer, intent(out) :: flag,hcnnz

  ! ARRAY ARGUMENTS
  integer,      intent(out) :: hccol(lim),hcrow(lim)
  real(kind=8), intent(in)  :: x(n)
  real(kind=8), intent(out) :: hcval(lim)

  call evalhc(n,x,set,ind,hcrow,hccol,hcval,hcnnz,lim,lmem,flag)
  

  end subroutine myevalhc

! ******************************************************************
! ******************************************************************

subroutine myevalfc(n,x,f,m,c,flag)

  implicit none

  ! SCALAR ARGUMENTS
  integer,      intent(in)  :: m,n
  integer,      intent(out) :: flag
  real(kind=8), intent(out) :: f

  ! ARRAY ARGUMENTS
  real(kind=8), intent(in)  :: x(n)
  real(kind=8), intent(out) :: c(m)

  flag = - 1

end subroutine myevalfc

! ******************************************************************
! ******************************************************************

subroutine myevalgjac(n,x,g,m,jcfun,jcvar,jcval,jcnnz,lim,lmem,flag)

  implicit none

  ! SCALAR ARGUMENTS
  logical,      intent(out) :: lmem
  integer,      intent(in)  :: lim,m,n
  integer,      intent(out) :: flag,jcnnz

  ! ARRAY ARGUMENTS
  integer,      intent(out) :: jcfun(lim),jcvar(lim)
  real(kind=8), intent(in)  :: x(n)
  real(kind=8), intent(out) :: g(n),jcval(lim)

  flag = - 1

end subroutine myevalgjac

! ******************************************************************
! ******************************************************************

subroutine myevalgjacp(n,x,g,m,p,q,work,gotj,flag)

  implicit none

  ! SCALAR ARGUMENTS
  logical,   intent(inout) :: gotj
  integer,   intent(in)    :: m,n
  integer,   intent(out)   :: flag
  character, intent(in)    :: work

  ! ARRAY ARGUMENTS
  real(kind=8), intent(in)    :: x(n)
  real(kind=8), intent(inout) :: p(m),q(n)
  real(kind=8), intent(out)   :: g(n)

  flag = - 1

end subroutine myevalgjacp

! ******************************************************************
! ******************************************************************

subroutine myevalhl(n,x,m,lambda,sf,sc,hlrow,hlcol,hlval,hlnnz,lim,lmem,flag)

  implicit none

  ! SCALAR ARGUMENTS
  logical,      intent(out) :: lmem
  integer,      intent(in)  :: lim,m,n
  integer,      intent(out) :: flag,hlnnz
  real(kind=8), intent(in)  :: sf

  ! ARRAY ARGUMENTS
  integer,      intent(out) :: hlcol(lim),hlrow(lim)
  real(kind=8), intent(in)  :: lambda(m),sc(m),x(n)
  real(kind=8), intent(out) :: hlval(lim)

  flag = - 1

end subroutine myevalhl

! ******************************************************************
! ******************************************************************

subroutine myevalhlp(n,x,m,lambda,sf,sc,p,hp,goth,flag)

  implicit none

  ! SCALAR ARGUMENTS
  logical,      intent(inout) :: goth
  integer,      intent(in)    :: m,n
  integer,      intent(out)   :: flag
  real(kind=8), intent(in)    :: sf

  ! ARRAY ARGUMENTS
  real(kind=8), intent(in)  :: lambda(m),p(n),sc(m),x(n)
  real(kind=8), intent(out) :: hp(n)

  flag = - 1

end subroutine myevalhlp




! ******************************************************************
! ******************************************************************

subroutine proj(n,set,y,w,z,m,l,u,equatn,linear,jcnnzmax,hcnnzmax,inform)

  use globals
  
  implicit none
  
  ! SCALAR ARGUMENTS
  integer, intent(in)  :: n,m,set,jcnnzmax,hcnnzmax
  integer, intent(out) :: inform
  
  ! ARRAY ARGUMENTS
  real(kind=8), intent(in)  :: y(n),w(n),l(n),u(n)
  real(kind=8), intent(out) :: z(n)
  logical,intent(in) :: equatn(m),linear(m)
  

  ! LOCAL SCALARS
  logical      :: checkder
  integer      :: allocerr,hnnzmax,nvparam,alinfo
  real(kind=8) :: cnorm,efacc,efstain,eoacc,eostain,epsfeas,epsopt, &
       f,nlpsupn,snorm

  ! LOCAL ARRAYS
  character(len=15)     :: strtmp
  character(len=80)     :: specfnm,outputfnm,vparam(10)
  logical               :: coded(11)
  real(kind=8), pointer :: lambda(:),x(:)

  ! EXTERNAL SUBROUTINES
  external :: myevalfexact,myevalgexact,myevalhexact,myevalc,myevaljac,myevalhc, &
              myevalfc,myevalgjac,myevalgjacp,myevalhl,myevalhlp
     
  ! inform:
  !
  ! 0: Feasibility, optimality and complementarity satisfied
  ! 2: Too large penalty parameter. Infeasible problem?
  ! 3: Maximum number of algencan iterations reached
  
  yinner   = y
  setinner = set

  ! Set lower bounds, upper bounds, and initial guess
  
  allocate(x(n),stat=allocerr)
  if ( allocerr .ne. 0 ) then
     write(*,*) 'Allocation error in main program'
     stop
  end if

  x = w

  ! Constraints
  
  allocate(lambda(m),stat=allocerr)
  if ( allocerr .ne. 0 ) then
     write(*,*) 'Allocation error in main program'
     stop
  end if

  lambda(1:m) = 0.0d0 
	
  ! Coded subroutines

  coded(1:6)  = .true.  ! fsub, gsub, hsub, csub, jacsub, hcsub
  coded(7:11) = .false. ! fcsub,gjacsub,gjacpsub,hlsub,hlpsub

  ! Upper bounds on the number of sparse-matrices non-null elements

  hnnzmax  = hcnnzmax + n
  
  ! Checking derivatives?

  checkder = .false.

  ! Parameters setting
  
  epsfeas   = 1.0d-08
  epsopt    = 1.0d-08

  efstain   = sqrt( epsfeas )
  eostain   = epsopt ** 1.5d0

  efacc     = sqrt( epsfeas )
  eoacc     = sqrt( epsopt )

  outputfnm = ''
  specfnm   = ''

  nvparam = 2
  vparam(1) = 'ITERATIONS-OUTPUT-DETAIL 00'
  vparam(2) = 'OUTER-ITERATIONS-LIMIT 100'
  

  ! Call Algencan

  ! E. G. Birgin and J. M. Martı́nez, Practical augmented Lagrangian 
  ! methods for constrained optimization, SIAM, 2014.
  
  call algencan(myevalfexact,myevalgexact,myevalhexact,myevalc,myevaljac,myevalhc, &
       myevalfc,myevalgjac,myevalgjacp,myevalhl,myevalhlp,jcnnzmax, &
       hnnzmax,epsfeas,epsopt,efstain,eostain,efacc,eoacc,outputfnm, &
       specfnm,nvparam,vparam,n,x,l,u,m,lambda,equatn,linear,coded, &
       checkder,f,cnorm,snorm,nlpsupn,inform,alinfo)
       
  inform = alinfo     
         
  z = x     

  deallocate(x,lambda,stat=allocerr)

  if ( allocerr .ne. 0 ) then
     write(*,*) 'Deallocation error in main program'
     stop
  end if

end subroutine proj

! ******************************************************************
! ******************************************************************

subroutine myevalfexact(n,x,f,flag)

  use globals, only: y => yinner

  implicit none

  ! SCALAR ARGUMENTS
  integer,      intent(in)  :: n
  integer,      intent(out) :: flag
  real(kind=8), intent(out) :: f

  ! ARRAY ARGUMENTS
  real(kind=8), intent(in) :: x(n)

  flag = 0	
      
  f = dot_product( x - y, x - y )
  f = f / 2.0d0

end subroutine myevalfexact

! ******************************************************************
! ******************************************************************

subroutine myevalgexact(n,x,g,flag)

  use globals, only: y => yinner

  implicit none

  ! SCALAR ARGUMENTS
  integer,      intent(in)  :: n
  integer,      intent(out) :: flag

  ! ARRAY ARGUMENTS
  real(kind=8), intent(in)  :: x(n)
  real(kind=8), intent(out) :: g(n)
  
  ! LOCAL SCALAR
  integer :: i

  flag = 0
    	
  g = x - y

end subroutine myevalgexact

! ******************************************************************
! ******************************************************************

subroutine myevalhexact(n,x,hrow,hcol,hval,hnnz,lim,lmem,flag)

  implicit none

  ! SCALAR ARGUMENTS
  logical,      intent(out) :: lmem
  integer,      intent(in)  :: lim,n
  integer,      intent(out) :: flag,hnnz

  ! ARRAY ARGUMENTS
  integer,      intent(out) :: hcol(lim),hrow(lim)
  real(kind=8), intent(in)  :: x(n)
  real(kind=8), intent(out) :: hval(lim)
  
  ! LOCAL SCALAR
  integer :: i

  flag = 0
  lmem = .false.

  hnnz = n
  
  if ( hnnz .gt. lim ) then
	 	lmem = .true.
	 	return 	
  end if
  
  do i = 1,n
	  hrow(i) = i
	  hcol(i) = i
	  hval(i) = 1.0d0
  end do
		
end subroutine myevalhexact

