module myproblem

use globals, only: problem,pi,beta

implicit none

  ! SUBROUTINES
  public :: inip,setparam,evalc,evaljac,evalhc,exactprojection

contains

subroutine inip(n,mA,xA,lA,uA,equatnA,linearA,optprojA,jcnnzmaxA,hnnzmaxA,&
          mB,xB,lB,uB,equatnB,linearB,optprojB,jcnnzmaxB,hnnzmaxB,erropt)

	implicit none
	
	! SCALAR ARGUMENTS
	integer,intent(out) :: n,mA,mB,optprojA,optprojB,jcnnzmaxA,        &
	                       jcnnzmaxB,hnnzmaxA,hnnzmaxB,erropt
	
	! ARRAY ARGUMENTS
	real(kind=8),allocatable,intent(out) :: xA(:),xB(:),lA(:),uA(:),lB(:),uB(:)
	logical,allocatable,intent(out) :: equatnA(:),linearA(:),equatnB(:),linearB(:)
	
	! LOCAL SCALARS
	integer :: i,allocerr
	
	
	if ( problem == 1 ) then 
	
		! Number of variables
	
		n = 2
		
		! Number of constraints (except bound constraints) of each set
		
		mA = 1
		mB = 1
		
		! Set the error option for the projection using de Condg routine
		
		erropt = 1
		
		! Set the projection option for each set
		! optprojX = 0: use a closed formula for the projection onto set 
		!               X (in this the subroutine exactprojection must 
		!               be coded for set X)
		! optprojX = 1: use an inexact projection onto set X by applying
		!               the Condition Gradient method 
		! optprojX = 2: use the exact projection onto set X by solving 
		!               the problem
		!               Minimize 1/2||y-x||**2 subject to x in X,
		!               for the projection of y onto X.
		
		optprojA = 1
		optprojB = 0
		
		allocate(xA(n),lA(n),uA(n),equatnA(mA),linearA(mA),            &
		         xB(n),lB(n),uB(n),equatnB(mB),linearB(mB),stat=allocerr)
		if ( allocerr .ne. 0 ) then
		   write(*,*) 'Allocation error in main program'
		   stop
		end if
		
		! Set the bounds constraints for the first set
		
		do i = 1,n
			lA(i) = -1.0d20
			uA(i) =  1.0d20
		end do
		
		! Set the bounds constraints for the second set
		
		do i = 1,n
			lB(i) = -1.0d20
			uB(i) =  1.0d20
		end do
		
		! Constraints: equality? linear?
		
		equatnA(1) = .false.
		linearA(1) = .false.
		
		equatnB(1) = .false.
		linearB(1) = .true.	
		
		! Upper bounds on the number of sparse-matrices non-null elements
		! jcnnzmax: number of non-null elements of the Jacobian
		! hnnzmax : number of non-null elements of the Hessians

		jcnnzmaxA = 2
		hnnzmaxA  = 3
		
		jcnnzmaxB = 1
		hnnzmaxB  = 0
		
		! Initial points
		
		xA(1) = 0.0d0
		xA(2) = 0.0d0

		xB(1) = 2.0d0
		xB(2) = 2.0d0			
		
		beta = 1.42d0
		
		
		return

	end if  
	
	! -----------------------------------------	
	
	if ( problem == 2 ) then 
	
		! Number of variables
	
		n = 2
		
		! Number of constraints (except bound constraints) of each set
		
		mA = 1
		mB = 1
		
		! Set the error option for the projection using de Condg routine
		
		erropt = 1
		
		! Set the projection option for each set
		! optprojX = 0: use a closed formula for the projection onto set 
		!               X (in this the subroutine exactprojection must 
		!               be coded for set X)
		! optprojX = 1: use an inexact projection onto set X by applying
		!               the Condition Gradient method 
		! optprojX = 2: use the exact projection onto set X by solving 
		!               the problem
		!               Minimize 1/2||y-x||**2 subject to x in X,
		!               for the projection of y onto X.
		
		optprojA = 1
		optprojB = optprojA
		
		allocate(xA(n),lA(n),uA(n),equatnA(mA),linearA(mA),            &
		         xB(n),lB(n),uB(n),equatnB(mB),linearB(mB),stat=allocerr)
		if ( allocerr .ne. 0 ) then
		   write(*,*) 'Allocation error in main program'
		   stop
		end if
		
		! Set the bounds constraints for the first set
		
		do i = 1,n
			lA(i) = -1.0d20
			uA(i) =  1.0d20
		end do
		
		! Set the bounds constraints for the second set
		
		do i = 1,n
			lB(i) = -1.0d20
			uB(i) =  1.0d20
		end do
		
		! Constraints: equality? linear?
		
		equatnA(1) = .false.
		linearA(1) = .false.
		
		equatnB(1) = .false.
		linearB(1) = .false.	
		
		! Upper bounds on the number of sparse Jacobian matrices non-null elements
		! jcnnzmax: number of non-null elements of the Jacobian
		! hnnzmax : number of non-null elements of the Hessians


		jcnnzmaxA = 2
		hnnzmaxA  = 3
		
		jcnnzmaxB = 2
		hnnzmaxB  = 3
		
		! Initial points
		
		xA(1) = 0.0d0
		xA(2) = 0.0d0
		
		xB(1) = 2.358d0
		xB(2) = 0.5d0			
		
		return

	end if   
	
end subroutine inip

! ******************************************************************
! ******************************************************************

subroutine setparam(a,b,angle,center,set)
	
	implicit none
	
	! SCALARS ARGUMENTS
	integer,intent(in) :: set
	real(kind=8),intent(out) :: a,b,angle
	
	! ARRAY ARGUMENTS
	real(kind=8),intent(out) :: center(2)
	
	if ( problem == 1 ) then 
	
		if ( set == 1 ) then
			a = 2.0d0
			b = 0.2d0
			
			angle = - pi / 4.0d0
			
			center(1) = 0.0d0
			center(2) = 0.0d0
			
			return
		end if
		
	end if
	
	! -----------------------------------------		

	
	if ( problem == 2 ) then 
	
		if ( set == 1 ) then
			a = 2.0d0
			b = 0.2d0
			
			angle = - pi / 4.0d0
			
			center(1) = 0.0d0
			center(2) = 0.0d0
			
			return
		end if
		
		if ( set == 2 ) then
			a = 2.0d0
			b = 0.4d0
			
			angle = pi / 3.0d0

			center(1) = 2.358d0
			center(2) = 0.5d0
			
			return
		end if
		
	end if	

end subroutine setparam

! ******************************************************************
! ******************************************************************

subroutine evalc(n,x,set,ind,c)
	
	implicit none

	! SCALAR ARGUMENTS
	integer,      intent(in)  :: n,set,ind
	real(kind=8), intent(out) :: c

	! ARRAY ARGUMENTS
	real(kind=8), intent(in)  :: x(n)
	
	! LOCAL SCALARS
	real(kind=8) :: a,b,angle
	
	! LOCAL ARRAYS
    real(kind=8) :: center(2) 
	
	if ( problem == 1 ) then 
	
		if ( set == 1 ) then
	
			call setparam(a,b,angle,center,set)
			
			c = ( cos(angle) * ( x(1) - center(1) ) + sin(angle) * ( x(2) - center(2) ) )**2/a**2 + &
			 (-sin(angle) * ( x(1) - center(1) ) + cos(angle) * ( x(2) - center(2) ) )**2/b**2 - 1.0d0
			
			return
		end if
		
		if ( set == 2 ) then
		
			c = - x(1) + beta
			
			return
		end if
	end if
	
! -----------------------------------------		
    	
	if ( problem == 2 ) then 
	
		call setparam(a,b,angle,center,set)
		
		c = ( cos(angle) * ( x(1) - center(1) ) + sin(angle) * ( x(2) - center(2) ) )**2/a**2 + &
		 (-sin(angle) * ( x(1) - center(1) ) + cos(angle) * ( x(2) - center(2) ) )**2/b**2 - 1.0d0
		
		return
	end if

end subroutine evalc

! ******************************************************************
! ******************************************************************

subroutine evaljac(n,x,set,ind,jcvar,jcval,jcnnz,lim,lmem,flag)

  implicit none

  ! SCALAR ARGUMENTS
  logical, intent(out) :: lmem
  integer, intent(in)  :: ind,set,lim,n
  integer, intent(out) :: flag,jcnnz

  ! ARRAY ARGUMENTS
  integer,      intent(out) :: jcvar(lim)
  real(kind=8), intent(in)  :: x(n)
  real(kind=8), intent(out) :: jcval(lim)
  
  ! LOCAL SCALARS
  real(kind=8) :: a,b,angle

  ! LOCAL ARRAYS
  real(kind=8) :: center(2)
  
  flag = 0
  lmem = .false.
  
  if ( problem == 1 ) then 
	
		if ( set == 1 ) then
  
			  jcnnz = 2
			  
			  if ( jcnnz .gt. lim ) then
				lmem = .true.
				return
			  end if
			  
			  call setparam(a,b,angle,center,set)
			  
			  jcvar(1) = 1
			  jcval(1) = 2.0d0 / a ** 2 * ( cos(angle) * ( x(1) - center(1) )  &
						+ sin(angle) * ( x(2) - center(2) ) ) * cos(angle)      &
						- 2.0d0 / b ** 2 * (-sin(angle) * ( x(1) - center(1) )  &
						+ cos(angle) * ( x(2) - center(2) ) ) * sin(angle) 
			  jcvar(2) = 2
			  jcval(2) = 2.0d0 / a ** 2 * ( cos(angle) * ( x(1) - center(1) ) &
						+ sin(angle) * ( x(2) - center(2) ) ) * sin(angle)    &
						+ 2.0d0 / b ** 2 * (-sin(angle) * ( x(1) - center(1) ) &
						+ cos(angle) * ( x(2) - center(2) ) ) * cos(angle) 
						
			  return
		end if
		
		if ( set == 2 ) then
				jcnnz = 1
				
				if ( jcnnz .gt. lim ) then
					lmem = .true.
					return
				end if
			  
			  jcvar(1) = 1
			  jcval(1) = -1.0d0
			  
			  return
		end if
  end if
  
  
 ! -----------------------------------------			
 
  
  if ( problem == 2 ) then 
  
	  jcnnz = 2
	  
	  if ( jcnnz .gt. lim ) then
		lmem = .true.
		return
	  end if
	  
	  call setparam(a,b,angle,center,set)
	  
	  jcvar(1) = 1
	  jcval(1) = 2.0d0 / a ** 2 * ( cos(angle) * ( x(1) - center(1) )  &
				+ sin(angle) * ( x(2) - center(2) ) ) * cos(angle)      &
				- 2.0d0 / b ** 2 * (-sin(angle) * ( x(1) - center(1) )  &
				+ cos(angle) * ( x(2) - center(2) ) ) * sin(angle) 
	  jcvar(2) = 2
	  jcval(2) = 2.0d0 / a ** 2 * ( cos(angle) * ( x(1) - center(1) ) &
				+ sin(angle) * ( x(2) - center(2) ) ) * sin(angle)    &
				+ 2.0d0 / b ** 2 * (-sin(angle) * ( x(1) - center(1) ) &
				+ cos(angle) * ( x(2) - center(2) ) ) * cos(angle) 
				
	  return
  end if
	
end subroutine evaljac

! ******************************************************************
! ******************************************************************

subroutine evalhc(n,x,set,ind,hcrow,hccol,hcval,hcnnz,lim,lmem,flag)
  
  implicit none

  ! SCALAR ARGUMENTS
  logical, intent(out) :: lmem
  integer, intent(in)  :: ind,set,lim,n
  integer, intent(out) :: flag,hcnnz

  ! ARRAY ARGUMENTS
  integer,      intent(out) :: hccol(lim),hcrow(lim)
  real(kind=8), intent(in)  :: x(n)
  real(kind=8), intent(out) :: hcval(lim)
  
  ! LOCAL SCALARS
  real(kind=8) :: a,b,angle

  ! LOCAL ARRAYS
  real(kind=8) :: center(2)

  flag = 0
  lmem = .false.
  
  if ( problem == 1 ) then
  
	if ( set == 1 ) then
  
		  hcnnz = 3
		  
		  call setparam(a,b,angle,center,set)
		  
		  hcrow(1) = 1
		  hccol(1) = 1 
		  hcval(1) = 2.0d0 * ( cos(angle) ** 2 / a ** 2 + sin(angle) ** 2 / b ** 2 )
		  
		  hcrow(2) = 2
		  hccol(2) = 1 
		  hcval(2) = 2.0d0 * sin(angle) * cos(angle) * ( 1.0d0 / a ** 2 - 1.0d0 / b ** 2 )
		   
		  hcrow(3) = 2
		  hccol(3) = 2 
		  hcval(3) = 2.0d0 * ( sin(angle) ** 2 / a ** 2 + cos(angle) ** 2 / b ** 2  )
		  
		  return
	end if
	
	if ( set == 2 ) then
		hcnnz = 0
	end if
	
  end if 
  
 ! -----------------------------------------			 
  
  if ( problem == 2 ) then 
  
	  hcnnz = 3
	  
	  call setparam(a,b,angle,center,set)
	  
	  hcrow(1) = 1
	  hccol(1) = 1 
	  hcval(1) = 2.0d0 * ( cos(angle) ** 2 / a ** 2 + sin(angle) ** 2 / b ** 2 )
	  
	  hcrow(2) = 2
	  hccol(2) = 1 
	  hcval(2) = 2.0d0 * sin(angle) * cos(angle) * ( 1.0d0 / a ** 2 - 1.0d0 / b ** 2 )
	   
	  hcrow(3) = 2
	  hccol(3) = 2 
	  hcval(3) = 2.0d0 * ( sin(angle) ** 2 / a ** 2 + cos(angle) ** 2 / b ** 2  )
	  
	  return
  end if 

  end subroutine evalhc

! ******************************************************************
! ******************************************************************

subroutine exactprojection(n,set,x,Px,flag)

  implicit none
  
  integer,intent(in) :: n,set
  real(kind=8),intent(in)  :: x(n)
  real(kind=8),intent(out) :: Px(n)
  integer,intent(out) :: flag


  if ( problem == 1 ) then
  
	if ( set == 1 ) then
		flag = -1
		return
	end if
	
	if ( set == 2 ) then
	
		flag = 0
		if ( x(1) >= beta ) then
			Px = x
		else
			Px(1) = beta
			Px(2) = x(2)
		end if
		return
	end if
	
  end if
  
  ! -----------------------------------------			
  
  if ( problem == 2 ) then 
	  flag = - 1
	  return
  end if

end subroutine exactprojection

end module myproblem
